package com.yixin.ms.service.impl;

import cn.hutool.core.lang.UUID;
import cn.hutool.core.util.*;
import com.alibaba.excel.util.DateUtils;
import com.alibaba.fastjson.JSON;
import com.yixin.ms.boot.restful.RestResponse;
import com.yixin.ms.boot.restful.RestResult;
import com.yixin.ms.boot.restful.ServiceException;
import com.yixin.ms.boot.uitls.Preconditions;
import com.yixin.ms.boot.uitls.RedisUtil;
import com.yixin.ms.model.dto.HealthCodeDTO;
import com.yixin.ms.model.dto.ShortTermDTO;
import com.yixin.ms.model.enums.*;
import com.yixin.ms.model.vo.HealtCodeNucleinVO;
import com.yixin.ms.model.vo.HealtCodeTypeVO;
import com.yixin.ms.model.vo.HealtCodeUserVO;
import com.yixin.ms.server.*;
import com.yixin.ms.server.vo.*;
import com.yixin.ms.service.HealthCodeService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.ParseException;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class HealthCodeServiceImpl implements HealthCodeService {

    @Autowired
    private RedisUtil redisUtil;
    @Autowired
    private PlaceHealthCodeServer placeHealthCodeServer;
    @Autowired
    private HealthCodeServer healthCodeServer;
    @Autowired
    private HealthServer healthServer;
    @Resource
    private HealthOpenReturnServer healthOpenReturnServer;

    @Override
    public HealtCodeUserVO getShortTermCertificate(ShortTermDTO shortTermDTO) {
        return healthServer.getShortTermCertificate(shortTermDTO);
    }

    @Override
    public String addToken(String token) {
        String s = "hh_token_" + UUID.randomUUID();
        boolean hanghui = redisUtil.set(s, s);
        if (hanghui) {
            return s;
        } else {
            throw new ServiceException("体检失败");
        }
    }

    @Override
    public Boolean deleteToken(String token) {
        return redisUtil.delete(token);
    }

    @Override
    public RestResult<HealtCodeUserVO> getShortTerm(ShortTermDTO shortTermDTO) {
        log.info("--->联动服务查询入参信息：{}", JSON.toJSONString(shortTermDTO));
        if (StringUtils.isBlank(shortTermDTO.getToken())) {
            throw new ServiceException(RestCodeEnum.FORBIDDEN.getCode(), RestCodeEnum.FORBIDDEN.getMsg());
        }

        /*查询设备账户信息 以及设备所绑账号的开关门信息*/
        PlaceInfoVO placeInfoVO = placeHealthCodeServer.queryAdminInfoBySn(shortTermDTO.getSn());
        if (Preconditions.isNotBlank(shortTermDTO.getHealthCode())) {//二维码
            if (Preconditions.isBlank(placeInfoVO.getOpenThirdLinkPlatform()) || !placeInfoVO.getOpenThirdLinkPlatform()) {
                throw new ServiceException(RestCodeEnum.FAIL.getCode(), "三方接口未开通无法解析二维码，请联系工作人员");
            }
        } else {//身份证
            if (Preconditions.isBlank(shortTermDTO.getIdNumber()) && Preconditions.isBlank(shortTermDTO.getName())) {
                throw new ServiceException(RestCodeEnum.PARAMS_IDCARD_INCOMPLETE.getCode(), RestCodeEnum.PARAMS_IDCARD_INCOMPLETE.getMsg());
            }
        }

        //判断是否是考试设备
        if (Preconditions.isNotBlank(placeInfoVO)
                && Preconditions.isNotBlank(placeInfoVO.getIsOpenExam())
                && placeInfoVO.getIsOpenExam() //是否开启考试
                && Preconditions.isNotBlank(placeInfoVO.getExamDevices())) {
            List<String> sns = Arrays.asList(placeInfoVO.getExamDevices().split(","));
            if (sns.contains(shortTermDTO.getSn())) {
                return examBiz(shortTermDTO, placeInfoVO);
            }
        }

        //初始化参数
        HealtCodeUserVO userVO = initHealthUserVo(shortTermDTO);

        // 处理姓名、身份证、手机号数据
        convertUserInfo(userVO, shortTermDTO);

        // 业务及用户信息、开关门校验及语音提示 （三方平台接口、黑名单、衢通码等）
        resultUserVOInfo(userVO, placeInfoVO, shortTermDTO);

        log.info("开门信号结果返回:{}", JSON.toJSONString(userVO));
        return RestResponse.ok(userVO);
    }


    /**
     * 补全返回参数数据
     * 1：用户信息
     * 2：开关门规则校验 及语音提示
     *
     * @param userVO
     * @param placeInfoVO
     * @param shortTermDTO
     */
    private void resultUserVOInfo(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, ShortTermDTO shortTermDTO) {
        //开关门规则校验 及语音提示
        healthOpenReturnServer.healthOpenReturn(userVO, placeInfoVO, shortTermDTO);
    }

    @Override
    public RestResult<HealthCodeVO> queryInfoByBarCode(HealthCodeDTO healthCodeDTO) {
        IrsTripAuthEnum authEnum = IrsTripAuthEnum.appKeyOf(healthCodeDTO.getAppKey());
        if (authEnum == null || !authEnum.getAppSecret().equals(healthCodeDTO.getAppSecret())) {
            return RestResponse.error("调用权限校验失败，请检查");
        }
        return healthCodeServer.queryInfoByBarCode(healthCodeDTO.getBarCode());
    }


    /**
     * 处理用户信息 身份证 姓名 手机号
     *
     * @param userVO
     * @param shortTermDTO
     */
    private void convertUserInfo(HealtCodeUserVO userVO, ShortTermDTO shortTermDTO) {
        //姓名和身份证号脱敏处理
        userVO.setUsername(Preconditions.isBlank(userVO.getUsername()) ? shortTermDTO.getName() : userVO.getUsername());
        userVO.setIdNumber(Preconditions.isBlank(userVO.getIdNumber()) ? shortTermDTO.getIdNumber() : userVO.getIdNumber());
        userVO.setShowIdNumber(Preconditions.isBlank(userVO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(userVO.getIdNumber(), 4, 4));
        userVO.setShowUsername(Preconditions.isBlank(userVO.getUsername()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(userVO.getUsername()));
        userVO.setPhone(Preconditions.isBlank(userVO.getPhone()) ? shortTermDTO.getPhone() : userVO.getPhone());
    }

    //查询考试信息
    private RestResult examBiz(ShortTermDTO shortTermDTO, PlaceInfoVO placeInfoVO) {
        if (StrUtil.isBlankIfStr(shortTermDTO.getIdNumber())) {
            throw new ServiceException(701, "身份证号不能为空");
        }
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        userVO.setIdNumber(StrUtil.isEmpty(shortTermDTO.getIdNumber()) ? StrUtil.EMPTY : shortTermDTO.getIdNumber());
        userVO.setShowIdNumber(StrUtil.isEmpty(shortTermDTO.getIdNumber()) ? StrUtil.EMPTY : DesensitizedUtil.idCardNum(shortTermDTO.getIdNumber(), 4, 4));
        userVO.setUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? StrUtil.EMPTY : shortTermDTO.getName());
        userVO.setShowUsername(StrUtil.isEmpty(shortTermDTO.getName()) ? StrUtil.EMPTY : DesensitizedUtil.chineseName(shortTermDTO.getName()));
        //查询
        RestResult<QueryExamUserWhitelistVO> result = placeHealthCodeServer.getExamUserWhitelist(shortTermDTO.getSn(), shortTermDTO.getIdNumber());
        initExamHealthInfo(userVO, placeInfoVO, result);
        return RestResponse.ok(userVO);
    }

    private void initExamHealthInfo(HealtCodeUserVO userVO, PlaceInfoVO placeInfoVO, RestResult<QueryExamUserWhitelistVO> result) {
        Integer isOpen = HealthOpenEnum.RELEASE.getCode();
        String codeTypeVOContent = "祝您考试顺利";
        //准考证号
        String travelValidation = StrUtil.EMPTY;
        //场次
        String content = StrUtil.EMPTY;
        //报考类别
        String vaccineNum = StrUtil.EMPTY;
        if (result != null && result.getData() != null) {
            travelValidation = result.getData().getExamCode();
            content = result.getData().getExamSession();
            vaccineNum = result.getData().getExamType();
        }

        String iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/green_tick.png";
        String travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/trip/green.png";
        if (result == null || result.getData() == null) {
            isOpen = HealthOpenEnum.NO_THOROUGHFARE.getCode();
            codeTypeVOContent = "未预约考试，禁止通行";
            iconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            vaccineIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
            travelValidationIconUrl = "https://noise.hz-hanghui.com:8088/yx-fyzd/file/icon/nucleic/exclamation_mark.png";
        }
        //开门信号
        userVO.setIsOpen(isOpen);
        //语音播报
        userVO.setOpenMsg(codeTypeVOContent != null ? codeTypeVOContent : "祝您考试顺利");
        userVO.setOpenVoiceMsg(codeTypeVOContent != null ? codeTypeVOContent : "祝您考试顺利");

        //全局背景
        userVO.setBackgroundColor(ColorEnum.GREEN.getValue());

        //头部
        String title = "温馨提醒"; //温馨提醒
        if (placeInfoVO != null
                && placeInfoVO.getTopTitle() != null) {
            title = placeInfoVO.getTopTitle();
        }
        userVO.setTitle(title == null ? "温馨提醒" : title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());

        //中上部
        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt(StrUtil.EMPTY);
        codeTypeVO.setContent(codeTypeVOContent);
        codeTypeVO.setContentColor(ColorEnum.GREEN.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));

        //中下部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        //左icon
        v.setIconUrl(iconUrl);
        v.setTitle("场次");
        v.setContent(content).setContentColor(ColorEnum.GREEN.getValue());
        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (Exception e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setResult(StrUtil.EMPTY);
        userVO.setNuclein(Arrays.asList(v));

        //右icon
        userVO.setVaccineIconUrl(vaccineIconUrl);
        userVO.setVaccineIcoTitle("报考类别");
        userVO.setVaccineNum(vaccineNum);
        userVO.setVaccineNumColor(ColorEnum.GREEN.getValue());

        //底部
        userVO.setTravelValidationIconUrl(travelValidationIconUrl);
        userVO.setTravelValidationTitle("准考证号");
        userVO.setTravelValidation(travelValidation);
        userVO.setTravelValidationDesc(StrUtil.EMPTY);
    }

    private HealtCodeUserVO initHealthUserVo(ShortTermDTO shortTermDTO) {
        HealtCodeUserVO userVO = new HealtCodeUserVO();
        //验证方式
        String vaccineNum = "身份证验证";
        if (Preconditions.isNotBlank(shortTermDTO.getHealthCode())) {
            vaccineNum = "二维码";
        }
        //全局背景色
        userVO.setBackgroundColor(ColorEnum.BLUE.getValue());
        //头部
        String title = "温馨提示";
        userVO.setTitle(title);
        userVO.setTitleColor(ColorEnum.WHITE.getValue());
        //中上部
        userVO.setVaccineIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        userVO.setVaccineNumColor("#2b6ef5");
        //中下左部
        HealtCodeNucleinVO v = new HealtCodeNucleinVO();
        v.setShowTime(null);
        Date date = null;
        String format = DateUtils.format(new Date(), DateUtils.DATE_FORMAT_19);
        try {
            date = DateUtils.parseDate(format, DateUtils.DATE_FORMAT_19);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        v.setChecktime(date);
        v.setReportTime(date);
        v.setIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        v.setTitle("温馨提示");
        v.setContent("携带好随身物品").setContentColor("#4e5c74");
        v.setResult(StrUtil.EMPTY);
        userVO.setNuclein(Arrays.asList(v));
        //中下右部
        userVO.setVaccineIcoTitle("验证方式");
        userVO.setVaccineNum(vaccineNum);
        //底部
        HealtCodeTypeVO codeTypeVO = new HealtCodeTypeVO();
        codeTypeVO.setMzt(StrUtil.EMPTY);
        codeTypeVO.setContent("请通行");
        codeTypeVO.setContentColor(ColorEnum.GREEN.getValue());
        userVO.setData(Arrays.asList(codeTypeVO));

        userVO.setTravelValidationTitle("动态行程卡");
        userVO.setTravelValidationIconUrl("https://tx.hz-hanghui.com:8088/yx-fyzd/file/icon/pass.png");
        userVO.setTravelValidationDesc(null);
        return userVO;
    }

}

